// Set up the SVG
const width = 1200;
const height = 800;
const svg = d3
  .select('#network-graph')
  .attr('width', width)
  .attr('height', height);

// Marker for arrowheads
svg
  .append('defs')
  .selectAll('marker')
  .data(['end'])
  .enter()
  .append('marker')
  .attr('id', String)
  .attr('viewBox', '0 -5 10 10')
  .attr('refX', 25)
  .attr('refY', 0)
  .attr('markerWidth', 3)
  .attr('markerHeight', 3)
  .attr('orient', 'auto')
  .append('path')
  .attr('d', 'M0,-5L10,0L0,5');

// Add title
svg
  .append('text')
  .attr('x', width / 2)
  .attr('y', 40)
  .attr('text-anchor', 'middle')
  .style('font-size', '28px')
  .style('font-weight', 'bold')
  .text('Filtered Category and Country Network Graph');

// Define the URL for fetching the data
const csvUrl =
  'https://raw.githubusercontent.com/nadipally-sai-snehith/Vis_Project/main/Global_Superstore2.csv';

// Color scale for the nodes
const color = d3.scaleOrdinal(d3.schemeCategory10);

// Fetch the CSV data
d3.csv(csvUrl).then((data) => {
  // Count occurrences of categories and countries
  let categoryCounts = {};
  let countryCounts = {};
  data.forEach((d) => {
    if (categoryCounts[d.Category]) {
      categoryCounts[d.Category] += 1;
    } else {
      categoryCounts[d.Category] = 1;
    }

    if (countryCounts[d.Country]) {
      countryCounts[d.Country] += 1;
    } else {
      countryCounts[d.Country] = 1;
    }
  });

  // Sort and select top categories and countries
  let topCategories = Object.keys(categoryCounts)
    .sort((a, b) => categoryCounts[b] - categoryCounts[a])
    .slice(0, 5);
  let topCountries = Object.keys(countryCounts)
    .sort((a, b) => countryCounts[b] - countryCounts[a])
    .slice(0, 10);

  // Filter data based on top categories and countries
  let filteredData = data.filter(
    (d) =>
      topCategories.includes(d.Category) &&
      topCountries.includes(d.Country),
  );
  // Nodes for top categories and countries
  const nodes = [
    ...topCategories.map((category) => ({
      id: category,
      group: 1,
    })),
    ...topCountries.map((country) => ({
      id: country,
      group: 2,
    })),
  ];

  // Links between top categories and countries
  const links = filteredData.map((d) => ({
    source: d.Category,
    target: d.Country,
  }));

  // Create the simulation
  const simulation = d3
    .forceSimulation(nodes)
    .force(
      'link',
      d3
        .forceLink(links)
        .id((d) => d.id)
        .distance(100),
    )
    .force('charge', d3.forceManyBody().strength(-50))
    .force('center', d3.forceCenter(width / 2, height / 2));

  // Draw the links
  const link = svg
    .append('g')
    .attr('class', 'links')
    .selectAll('line')
    .data(links)
    .enter()
    .append('line')
    .attr('stroke', '#aaa')
    .attr('stroke-width', 2)
    .attr('marker-end', 'url(#end)');

  // Draw the nodes
  const node = svg
    .append('g')
    .attr('class', 'nodes')
    .selectAll('circle')
    .data(nodes)
    .enter()
    .append('circle')
    .attr('r', 5)
    .attr('fill', (d) => color(d.group))
    .call(
      d3
        .drag()
        .on('start', dragstarted)
        .on('drag', dragged)
        .on('end', dragended),
    );

  // Node labels
  const label = svg
    .append('g')
    .attr('class', 'labels')
    .selectAll('text')
    .data(nodes)
    .enter()
    .append('text')
    .attr('x', 8)
    .attr('y', '.31em')
    .text((d) => d.id);

  // Add tick instructions
  simulation.on('tick', () => {
    link
      .attr('x1', (d) => d.source.x)
      .attr('y1', (d) => d.source.y)
      .attr('x2', (d) => d.target.x)
      .attr('y2', (d) => d.target.y);

    node.attr('cx', (d) => d.x).attr('cy', (d) => d.y);

    label.attr('x', (d) => d.x).attr('y', (d) => d.y);
  });

  // Drag functions
  function dragstarted(event, d) {
    if (!event.active)
      simulation.alphaTarget(0.3).restart();
    d.fx = d.x;
    d.fy = d.y;
  }

  function dragged(event, d) {
    d.fx = event.x;
    d.fy = event.y;
  }

  function dragended(event, d) {
    if (!event.active) simulation.alphaTarget(0);
    d.fx = null;
    d.fy = null;
  }
});
